/*  
  classMainWindow
  Developed on 2010 03 25
  Author R.G. care of Datalogic Automation Laser Marking Division
*/

/***********************************************************
  Internal Tables
***********************************************************/
const LINES_LOG=3;
const hIndex=0;
const getCurrentDistance=false;
var onlyTrigger=false;

const OK = "OK";
const WAIT = "Wait";
const WARNING="Warning";
const markWhenFocusReached="markWhenFocusReached";
const loadedLayout="loadedLayout";

const ParityMap = [ [ComPort.PAR_NONE, "None"],
    [ComPort.PAR_ODD, "Odd"],
    [ComPort.PAR_EVEN , "Even"],
    [ComPort.PAR_MARK , "Mark"],
    [ComPort.PAR_SPACE , "Space"] ];
const BaudsMap = [ [ComPort.BAUD110, "B 110"],
    [ComPort.BAUD300, "B 300"],
    [ComPort.BAUD600, "B 600"],
    [ComPort.BAUD1200, "B 1200"],
    [ComPort.BAUD2400, "B 2400"],
    [ComPort.BAUD4800, "B 4800"],
    [ComPort.BAUD9600, "B 9600"],
    [ComPort.BAUD14400, "B 14400"],
    [ComPort.BAUD19200, "B 19200"],
    [ComPort.BAUD38400, "B 38400"],
    [ComPort.BAUD56000, "B 56000"],
    [ComPort.BAUD57600, "B 57600"],
    [ComPort.BAUD115200, "B 115200"],
    [ComPort.BAUD128000, "B 128000"],
    [ComPort.BAUD256000, "B 256000"] ];
const DataBitsMap = [ [ComPort.DATA_5, "b 5"],
    [ComPort.DATA_6, "b 6"],
    [ComPort.DATA_7, "b 7"],
    [ComPort.DATA_8, "b 8"] ];
const StopBitsMap = [ [ComPort.STOP_1, "b 1"],
    [ComPort.STOP_1_5, "b 1.5"],
    [ComPort.STOP_2, "b 2"] ];
const FlowMap = [ [ComPort.FLOW_OFF, "Off"],
    [ComPort.FLOW_HARDWARE, "HW"],
    [ComPort.FLOW_XONXOFF, "Xon / Xoff"] ];

/***********************************************************
  Class classMainWindow Definition
***********************************************************/
classMainWindow.prototype = new Dialog("Datalogic S.r.l.", Dialog.D_NONE, false);

function classMainWindow(receiver)
{
    this.InReceiver = receiver;
    
      this.lbl = new Array(2);
  
  for (i = 0; i < this.lbl.length; ++i)
  {
    this.lbl[i] = new Label();
	this.lbl[i].alignment = 0x04;
    this.lbl[i].text = "";
  }
  this.lbl[0].text = AppTitle;
  
  this.gbMarkMode = new GroupBox();
  this.cbAutoManual = new CheckBox();
  this.cbAutoManual.text="Start Marking when Focal Distance is reached";
  this.cbAutoManual.checked=true;
  this.cbAutoManual.sigStateChanged.connect(updateGlobalVariable);
  this.gbMarkMode.add(this.cbAutoManual);
  this.gbMarkMode.newColumn();
  this.pbManualMarking = new PushButton();
  this.pbManualMarking.setFixedSize(200,25);
  this.pbManualMarking.text="Start Marking";
  this.pbManualMarking.sigReleased.connect(onMark);
  this.gbMarkMode.add(this.pbManualMarking);
  this.add(this.gbMarkMode);
  
  this.cmb = new Array(6);
  for (i = 0; i < this.cmb.length; ++i)
    this.cmb[i] = new ComboBox();
  
  this.cmb[0].label = "Port";
  var list = [];
  for (i=1; i<6; ++i)
    list.push("COM" + i);
  this.cmb[0].itemList = list;
  this.cmb[0].currentItem = h_port.instance.portName;
  
  this.cmb[1].label = "BaudRate";
  var list = [];
  for (i=0; i<BaudsMap.length; ++i)
    list.push(BaudsMap[i][1]);
  this.cmb[1].itemList = list;
  this.cmb[1].currentItem = lookUp(BaudsMap, h_port.instance.baudRate);
  
  this.cmb[2].label = "Data Bits";
  var list = [];
  for (i=0; i<DataBitsMap.length; ++i)
    list.push(DataBitsMap[i][1]);
  this.cmb[2].itemList = list;
  this.cmb[2].currentItem = lookUp(DataBitsMap, h_port.instance.dataBits);
  
  this.cmb[3].label = "Parity";
  var list = [];
  for (i=0; i<ParityMap.length; ++i)
    list.push(ParityMap[i][1]);
  this.cmb[3].itemList = list;
  this.cmb[3].currentItem = lookUp(ParityMap, h_port.instance.parity);
  
  this.cmb[4].label = "Stop";
  var list = [];
  for (i=0; i<StopBitsMap.length; ++i)
    list.push(StopBitsMap[i][1]);
  this.cmb[4].itemList = list;
  this.cmb[4].currentItem = lookUp(StopBitsMap, h_port.instance.stopBits);
  
  this.cmb[5].label = "Flow Control";
  var list = [];
  for (i=0; i<FlowMap.length; ++i)
    list.push(FlowMap[i][1]);
  this.cmb[5].itemList = list;
  this.cmb[5].currentItem = lookUp(FlowMap, h_port.instance.flowControl);
    
    var varState = System.getStringValue(markWhenFocusReached);
  if((varState.match!=("ERR")) && (varState==1)) {  //if the global variable exists and it is =1
      this.cbAutoManual.checked=true;
  }
  else this.cbAutoManual.checked=false;
  
    this.gbLayout = new GroupBox();
    this.gbLayout.title="Layout selection";
    this.cbDocs = new ComboBox();
    this.cbDocs.label="Layout: ";
    DocsDir=new Dir (System.getDocumentsDir());
    this.cbDocs.itemList = DocsDir.entryList("*.xlp");
    //loading the last *.xlp which was selected by the user, if present
    var varLayout = System.getStringValue(loadedLayout);
    if(varState.match!=("ERR")) {  //if the global variable exists
	print("The last loaded layout is "+varLayout);
	this.cbDocs.currentItem=varLayout;
    }
    else print("A layout has never been selected on this Laser Marker through this script");
    
    this.cbDocs["sigIndexChanged(QString)"].connect(this, "onIndexChanged");
    this.gbLayout.add(this.cbDocs);
    
      this.btn = new Array(2);
  for (i = 0; i < this.btn.length; ++i)
  {
    this.btn[i] = new PushButton();
	this.btn[i].setFixedSize(120,20);
	//this.add(this.btn[i]);
  }
  this.btn[0].text = "Open";
  this.btn[1].text = "Close";
    
    this.prv = new RenderArea();
    this.gbLayout.add(this.prv);
    
    this.add(this.gbLayout);
    
    this.msgLabel0 = new Label();
    this.msgLabel1 = new Label();
    this.msgLabel2 = new Label();
    this.msgLabel3 = new Label();
    this.msgLabel4 = new Label();
    this.lbTcpPort = new Label();
    
    this.msgLabel0.alignment = 0x04;
    this.msgLabel1.alignment = 0x04;
    this.msgLabel2.alignment = 0x04;
    this.msgLabel3.alignment = 0x04;
    this.msgLabel4.alignment = 0x04;
    this.lbTcpPort.alignment = 0x04;
    
    this.msgLabel0.text = "";
    this.msgLabel1.text = "";
    this.msgLabel2.text = "";
    this.msgLabel3.text = "";
    this.msgLabel4.text = AppTitle;    
    this.lbTcpPort.text = "";
    
    this.GroupBase = new GroupBox();
  /*this.GroupBase.title = "COM parameters";   //adding the serial communication settings to the GUI
  this.GroupBase.addSpace(5);
  this.GroupBase.add(this.cmb[0]);
  this.GroupBase.add(this.cmb[1]);
  this.GroupBase.add(this.cmb[2]);
  this.GroupBase.newColumn();
  this.GroupBase.addSpace(5);
  this.GroupBase.add(this.cmb[3]);
  this.GroupBase.add(this.cmb[4]);
  this.GroupBase.add(this.cmb[5]);*/
    this.GroupBase.title = "Outcome";
    this.lblResult = new Label();
  
    this.add(this.GroupBase);
    this.GroupBase.add(this.lblResult);
    this.GroupBase.newColumn();
    this.lblText = new Label();
    this.GroupBase.add(this.lblText);
    
    this.gbLastReport = new GroupBox();
    this.gbLastReport.title= "Last acquisition";
    this.add(this.gbLastReport);
    this.leDistance = new LineEdit();
    this.leDistance.label = "Distance [mm]: "
    this.leDistance.enable=false;
    this.gbLastReport.add(this.leDistance);
    this.gbLastReport.newColumn();
    this.PBtrigger = new PushButton;
    this.PBtrigger.text="TRIGGER";
    this.PBtrigger.setFixedSize(200,25);
    this.PBtrigger["sigReleased()"].connect(this, "onTrigger");
    this.gbLastReport.add(this.PBtrigger);
    
    
    this.gbButtons = new GroupBox();
    this.PBsaveDistance = new PushButton;
    this.PBsaveDistance.text="SAVE CURRENT DISTANCE";
    this.PBsaveDistance.setFixedSize(200,50);
    this.PBsaveDistance["sigReleased()"].connect(this, "onSaveDistance");
    this.gbButtons.add(this.PBsaveDistance);
    this.gbButtons.newColumn();
    this.PBfocusStart = new PushButton;
    this.PBfocusStart.text="START AUTOFOCUS";
    this.PBfocusStart.setFixedSize(200,50);
    this.PBfocusStart["sigReleased()"].connect(this, "onFocusStart");
    this.PBfocusStart.enable=false;
    this.gbButtons.add(this.PBfocusStart);
    this.add(this.gbButtons);
    
    /*this.PBstepUp = new PushButton;
    this.PBstepUp.text="STEP UP";
    this.PBstepUp.setFixedSize(100,50);
    this.PBstepUp["sigReleased()"].connect(onStepUp);
    this.add(this.PBstepUp);
    
    this.PBstepDown = new PushButton;
    this.PBstepDown.text="STEP DOWN";
    this.PBstepDown.setFixedSize(100,50);
    this.PBstepDown["sigReleased()"].connect(onStepDown);
    this.add(this.PBstepDown);    */
    
    this.lblLog = new Array(LINES_LOG);
    for (var i=0;i<this.lblLog.length;++i) {
	this.lblLog[i] = new Label();
	this.add(this.lblLog[i]);
    }
    this.show();
}

function updateGlobalVariable(state) {
    print("updateGlobalVariable - state:"+state);
    if (state==2) {   //the checkbox has just been checked
	System.makeStringVariable(markWhenFocusReached, "1", true);
    }
    else {   //the checkbox has been disabled
	System.makeStringVariable(markWhenFocusReached, "0", true);
    }
}

classMainWindow.prototype.showMessage = function( target, message )
{
  this.lbl[target].text = message;
}

classMainWindow.prototype.onIndexChanged = function () {
    print(this.cbDocs.itemList.length +" files detected")
	    print("result: "+hDoc.load( this.cbDocs.currentItem));
    System.makeStringVariable(loadedLayout, this.cbDocs.currentItem, true);
    var ids = hDoc.getObjectIDs().split(",");
    print("ids: "+ids);
    this.prv.preview(hDoc);
}
classMainWindow.prototype.onFocusStart = function() {    
    print("onFocusStart");
    log("Autofocus Started");
    savedPosition=Axis.getPosition(2).toFixed(2);
    print("savedPosition: "+savedPosition);
    measureHeight=true;
    if (h_Proto.protocolSend("{0M}")) {
	print("sentMessage");
    }	
}
function log (message) {   //this function organizes the messages on the Dialog window
   var d = new Date();
   message = d.toLocaleTimeString() +" - " + message;   
   if(hIndex<LINES_LOG) {
       h_Window.lblLog[hIndex].text=message;  
       hIndex++;
   }
   else {
       for (var j = 1; j < LINES_LOG; ++j)   //label shift cycle upwards
       h_Window.lblLog[j - 1].text = h_Window.lblLog[j].text;
       h_Window.lblLog[h_Window.lblLog.length - 1].text = message;
   }
   print(message);
}

classMainWindow.prototype.onSaveDistance = function() {
    getCurrentDistance=true;
    savedPosition=Axis.getPosition(2);
    h_Proto.protocolSend("{0M}");
    this.PBfocusStart.enable=true;
}

classMainWindow.prototype.OK = function () {
    this.lblResult.loadImage(System.getProjectDir() + "/" + OK,20,20,2);
    this.lblText.text="AXIS IN POSITION";
}

classMainWindow.prototype.Wait = function () {
    this.lblResult.loadImage(System.getProjectDir() + "/" + WAIT,20,20,2);
    this.lblText.text="MOVING AXIS INTO POSITION";
}

classMainWindow.prototype.Warning = function () {
    this.lblResult.loadImage(System.getProjectDir() + "/" + WARNING,20,20,2);
    this.lblText.text="FOCUS SENSOR OUT OF RANGE";
}

classMainWindow.prototype.destinationOutOfRange = function () {
    this.lblResult.loadImage(System.getProjectDir() + "/" + WARNING,20,20,2);
    this.lblText.text="CANNOT MOVE OUTSIDE Z AXIS RANGE";
}

classMainWindow.prototype.CommunicationError = function () {
    this.lblResult.loadImage(System.getProjectDir() + "/" + WARNING,20,20,2);
    this.lblText.text="COMMUNICATION ERROR";
}

classMainWindow.prototype.autofocusNotCompletedCorrectly = function () {
    this.lblResult.loadImage(System.getProjectDir() + "/" + WARNING,20,20,2);
    this.lblText.text="AUTOFOCUS HAS NOT BEEN COMPLETED CORRECTLY";
}

classMainWindow.prototype.onTrigger = function() {
    onlyTrigger=true;
        if (h_Proto.protocolSend("{0M}")) {
	print("sentMessage");
    }
}