// Protocol definitions
const ETX  = 125;
const STX = 27;
const SEPARATOR = ",";
const ENDOFMARK = 7;
const CHAR_T = 84;
const TIMER=250;  //ms

/***********************************************************
  The protocol
***********************************************************/
function classProtocol()
{
 this.message = [];   
}

classProtocol.prototype.start = function( port, dlg)
{
  trace("classProtocol::start");
  this.port = port;
  this.port.update( dlg );
  this.port.instance.ETX = ETX;
  this.port.instance["readyRead()"].connect( this, "MessageReady" );
  if (this.port.open(ComPort.ReadWrite))
    log("Port " + port.instance.portName + " opened succesfully!");
  else
    log("An error occurred opening port " + port.instance.portName + " !");
}

classProtocol.prototype.stop = function()
{
  this.port.instance.close();
  log("Port " + this.port.instance.portName + " closed!");
}

classProtocol.prototype.MessageReady = function()
{
  do
  {	  
    this.message = this.message.concat(this.port.instance.readBlock());
   }  
  while (this.port.instance.bytesAvailable > 0)
  
  // Check frame
  var found, esc, count = this.message.length;
  trace("Received(" + count + "): " + this.message);
  if(this.message[count-1] !=0x7d)
      return;
	  
  for (found = true, esc = -1; found; )
  {
    esc = find(this.message, ETX, esc + 1);
    trace("esc=" + esc);
    if (esc >= 0)
    {
      found = true;
      var mex=arrayOfIntToString(this.message);
      log("Received: '"+mex+"'");
      elaborateDatagram(mex);
    }
    else
      found = false;
  }
  
  while( this.message.length)
      this.message.pop();
   trace("Clear buffer: " + this.message);
}

var delta=0;

classProtocol.prototype.protocolSend = function(command)
{
this.port.instance.flush();
if(this.port.instance.writeBlock(command, command.length)==command.length) {
    print("command sent");  
    System["sigTimer(int)"].connect(onTimer);
    timer1 = System.setTimer(2000);
}

else print("command not sent");
}

/********************************************************************************/
function onTimer(ID)
{
    if ( timer1 == ID) {
	System.killAllTimers();
	System["sigTimer(int)"].disconnect(onTimer);
	print( "No reply received in 2 seconds");
	h_Window.CommunicationError();
	log("COMMUNICATION ERROR: Sensor is not replying");
	
    }

}

var d=undefined;

var searchingForFiniteValue=false;

function elaborateDatagram (message) {
    System.killAllTimers();
    var currentHeight=(message.slice(4,-8))/100;
    currentHeight=currentHeight.toFixed(2);
    print("currentHeight: "+currentHeight);
    h_Window.leDistance.text=currentHeight;
	if(setUpSensorPhase) {//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    switch(step) {
	    case 0:   //setting the distance sensor to 0,01 mm
		if(message=="{0SH03}") {   //ACK
		    step=1;
		    setupSensor(true);
		}
		else {   //NACK
		    setupSensor(false);
		}
		break;
	    case 1:    //setting distance and attenuation
		if(message=="{0ZMA80}") {   //ACK
		    step=2;
		    setupSensor(true);
		}
		else {   //NACK
		    setupSensor(false);
		}
		break;
	    case 2:   //save Laser configuration
		if(message=="{0K23}") {   //ACK
		    step=3;
		    setupSensor(true);
		    setUpSensorPhase=false;
		}
		else {   //NACK
		    setupSensor(false);
		}
		break;
	    default:
		log("GENERIC ERROR");
	    }
	}
	else {
	    	    if ((getCurrentDistance)&&(currentHeight!=0)&&(currentHeight!=999.99)) {
			h_Window.OK();
			d=currentHeight;
			log("Focal distance [mm] set at:  "+d);
			System.makeStringVariable(DISTANCE, d, true);
			getCurrentDistance=false;
		    }
		    else {
			//report the measured height in the GUI
	
			//if((parseInt(currentHeight)-99999!=0)&&(parseInt(currentHeight)-1!=-1)) {   //enters here if the Sensor returns a sensible value, different from '00000' or '99999'
			    if((currentHeight!=999.99)&&(currentHeight!=0)) {
			    h_Window.Wait();
			    Axis.stop(2);
			    searchingForFiniteValue=false;
			    print("Distance understood");
			    print("d: "+d+", currentHeight: "+currentHeight+", savedPosition: "+savedPosition);
			    //var newPos=savedPosition+d-parseInt(currentHeight, 10);
			    delta=parseFloat(currentHeight-d).toFixed(2);			    
			    var newPos=(parseFloat(savedPosition)+(d-currentHeight)).toFixed(2);
			    print("newPos: "+newPos);
			    print("Absolute value delta: "+Math.abs(delta));
			    if(lastCheck) {
				lastCheck=false;
				if(Math.abs(delta)<TOLERANCE) {
				    h_Window.OK();
				    log("Autofocus has terminated successfuly");
				    if(h_Window.cbAutoManual.checked) onMark();
				}
				else {
				    h_Window.autofocusNotCompletedCorrectly();
				    log("Autofocus has not been completed successfully");
				}
			    }		    
			    else if(Math.abs(delta)<TOLERANCE) {
				log("The height which has to be reached is currently within the "+TOLERANCE+" mm Tolerance");
				autofocusOngoing=false;
				h_Window.OK();
				log("Autofocus has terminated successfuly");
				if(h_Window.cbAutoManual.checked) onMark();
			    }
			    else if ((m<=newPos)&&(newPos<=M)) {       //if the final position which need to be reached  in inside the Z axis range
				autofocusOngoing=true;
				print("newPos: "+newPos+", delta: "+(d-currentHeight));
				print("Moving Z axis: "+Axis.moveMoreAxis(0x04, 0, 0, newPos, 0));
			    }
			    else {
				log ("Cannot move to position "+newPos+" as it is out of Z axis range ["+m+", "+M+"]");
				autofocusOngoing=false;
				h_Window.destinationOutOfRange();
			    }

			}
			else {
			    h_Window.Warning();
			    autofocusOngoing=false;
			    log("OUT OF RANGE or possible reflection of the Focusing Beam");   //enters here when one of these non-sensible values have been returned by the sensor
			    h_Window.leDistance.text="-"
			if(onlyTrigger) {
				onlyTrigger=false;
			    }
			}
		    }	
		}

}

function setupSensor(previousState) {   //this function must be responsible for the management of the setup of the Distance Sensor, which includes (1) setting the Sensitivity of the Distance Sensor to 0,01mm, (2) requesting the sensor to return both distance and attenuation, (3) Saving the configuration.
    print("setupSensor("+previousState+") - step: "+step);
    if(previousState==true) {
	switch (step) {
	case 0:
	    if (h_Proto.protocolSend("{0SH}")) {
		print("step 0 - sentMessage");
	    }
	    break;
	case 1: 
	    if (h_Proto.protocolSend("{0ZMA}")) {
		print("step 1 - sentMessage");
	    }
	    break;
	case 2:
	    if (h_Proto.protocolSend("{0K}")) {
		print("step 2 - sentMessage");
	    }
	    break;
	case 3:
	    log("Distance Sensor set up: COMPLETED");
	    //setUpSensorPhase=false;
	    break;
	default:
	    log("GENERIC ERROR");
	}	
    }
    else {
	h_Window.PBtrigger.enable=false;
	h_Window.PBfocusStart.enable=false;
	log("Distance Sensor set up: FAILED - please restart the script");
    }

}

function triggerDistanceSensor (stage) {
    print("triggerDistanceSensor - stage: "+stage);
    switch(step) {
    case 0: 
	
	break;
    case 1: 

	break;
    case 2:
	
	break;
    default:
	log ("GENERIC ERROR");
    }
    	    if (h_Proto.protocolSend("{0M}")) {
		print("sentMessage");
	    }	
}